@@ -216,13 +216,19 @@ class Message:
     request_id: str
     data: bytes
 
+    def get_str(self) -> str:
+        return self.data.decode()
+    
+    def get_obj(self) -> dict | list | int | float | bool | str | None:
+        return json.loads(self.get_str())
+
 
 class Client:
     def __init__(
             self, 
             host: str, 
             port: int, 
-            on_receive: callable,
+            on_receive: Callable[[Message], Awaitable[None] | None],
         ) -> None:
         # Initialize encryption service
         self.client_id = str(uuid.uuid4())
@@ -266,6 +272,9 @@ class Client:
         send(self.send_conn, self.encrypt(SPLIT_TOKEN.join([method.encode(), request_id.encode(), data])))
         receive(self.send_conn)
         return request_id
+    
+    def send_obj(self, method: str, data: dict | list | int | float | bool | str | None) -> str:
+        return self.send(method, json.dumps(data).encode())
 
     def __receive_thread(self) -> bytes:
         while self.receiving:
@@ -372,6 +381,9 @@ class Client:
         await async_send(self.send_conn, await self.aencrypt(SPLIT_TOKEN.join([method.encode(), request_id.encode(), data])))
         await async_receive(self.send_conn)
         return request_id
+    
+    async def asend_obj(self, method: str, data: dict | list | int | float | bool | str | None) -> str:
+        return await self.asend(method, json.dumps(data).encode())
 
     async def __areceive_thread(self) -> bytes:
         while self.receiving:
@@ -473,10 +485,15 @@ class Client:
 
 @dataclass
 class ServerRequest:
+    client_id: str
+    request_id: str
     method: str
     data: bytes
     send_data: Callable[[bytes], None]
 
+    def send(self, data: str) -> None:
+        self.send_data(data.encode())
+
 
 class Server:
     def __init__(self, host: str, port: int, handler: Callable[[ServerRequest], None | Awaitable[None]]):
@@ -529,7 +546,7 @@ class Server:
             pass
         return data
     
-    def handle_requests(self, s, q: queue.Queue, request_q: queue.Queue, loop=None):
+    def handle_requests(self, client_id: str, s, q: queue.Queue, request_q: queue.Queue, loop=None):
         method, request_id, data = self.decrypt(self.server_receive(s)).split(SPLIT_TOKEN)
         send(s, b'ok')
 
@@ -541,6 +558,8 @@ class Server:
                 
             request_q.put_nowait(
                 ServerRequest(
+                    client_id,
+                    request_id.decode(),
                     method.decode(), 
                     data, 
                     get_send_data_func(request_id)
@@ -577,7 +596,7 @@ class Server:
                 #     self.handler(ServerRequest(method.decode(), data, send_data))
                 #     method, request_id, data = self.decrypt(self.server_receive(s)).split(SPLIT_TOKEN)
 
-                t = threading.Thread(target=self.handle_requests, args=(s, q, request_q), daemon=True)
+                t = threading.Thread(target=self.handle_requests, args=(client_id, s, q, request_q), daemon=True)
                 t.start()
 
                 while (request := request_q.get()) is not None:
@@ -656,7 +675,7 @@ class Server:
             pass
         return data
     
-    async def ahandle_requests(self, s, q: asyncio.Queue, request_q: asyncio.Queue):
+    async def ahandle_requests(self, client_id: str, s, q: asyncio.Queue, request_q: asyncio.Queue):
         method, request_id, data = (await self.adecrypt(await self.aserver_receive(s))).split(SPLIT_TOKEN)
 
         while method != b'close':
@@ -667,7 +686,7 @@ class Server:
                     q.put_nowait(SPLIT_TOKEN.join([req_id, data]))
                 return send_data
                 
-            await request_q.put(ServerRequest(method.decode(), data, get_send_data_func(request_id)))
+            await request_q.put(ServerRequest(client_id, request_id.decode(), method.decode(), data, get_send_data_func(request_id)))
             # self.handler(ServerRequest(method.decode(), data, send_data))
             method, request_id, data = (await self.adecrypt(await self.aserver_receive(s))).split(SPLIT_TOKEN)
         
@@ -689,7 +708,7 @@ class Server:
                 await async_send(s, self.encrypt(b'ok'))
 
                 s.setblocking(True)
-                t = threading.Thread(target=self.handle_requests, args=(s, q, request_q, asyncio.get_running_loop()), daemon=True)
+                t = threading.Thread(target=self.handle_requests, args=(client_id, s, q, request_q, asyncio.get_running_loop()), daemon=True)
                 t.start()
                 # asyncio.create_task(self.ahandle_requests(s, q, request_q))
 
@@ -739,3 +758,15 @@ def server_handler_example(request: ServerRequest) -> None:
 #     if request.method == 'echo':
 #         request.send_data(request.data)
 
+
+BiServer = Server
+BiClient = Client
+BiMessage = Message
+BiServerRequest = ServerRequest
+
+
+
+
+
+
+
